package com.tzld.piaoquan.recommend.server.service.filter;

import com.tzld.piaoquan.recommend.server.service.PreViewedService;
import com.tzld.piaoquan.recommend.server.service.ViewedService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
public abstract class FilterService {
    @Autowired
    private PreViewedService preViewedService;

    @Autowired
    private ViewedService viewedService;

    public FilterResult filter(FilterParam param) {
        List<Long> videoIds = filterByPreViewed(param.getAppType(), param.getMid(), param.getVideoIds());
        if (CollectionUtils.isEmpty(videoIds)) {
            return new FilterResult(videoIds);
        }
        videoIds = filterByViewed(param.getAppType(), param.getMid(), param.getUid(), videoIds);

        return new FilterResult(videoIds);
    }

    protected List<Long> filterByViewed(int appType, String mid, String uid, List<Long> videoIds) {
        // TODO uid为空时，还需要过滤么？
        if (StringUtils.isBlank(mid)) {
            return videoIds;
        }
        return viewedService.filterViewedVideo(appType, mid, uid, videoIds);

    }

    protected List<Long> filterByPreViewed(int appType, String mid, List<Long> videoIds) {

        if (StringUtils.isBlank(mid)) {
            return videoIds;
        }
        Set<Long> preViewedVideoIds = preViewedService.getVideoIds(appType, mid);
        return videoIds.stream()
                .filter(l -> !preViewedVideoIds.contains(l))
                .collect(Collectors.toList());

    }
}
